package fr.umlv.foogle;

import java.io.File;
import java.io.IOException;
import java.io.Reader;

import java.util.Map;
import java.util.regex.Pattern;

/** Access point for creating, loading, saving foogle index and
 *  performs some queries on it.
 * 
 * @author Remi Forax
 */
public interface FoogleFacade {
  /** Creates an in-memory index with a specified strategy
   *  and appends all words extracted of a specific file.
   * @param strategy the context strategy used to create the index.
   * @param file all words of the file will be appended in the index. 
   * @return a new in-memory index.
   * @throws IOException if the file doesn't exists or is unreadable.
   */
  public Index createIndex(File file,ContextStrategy strategy) throws IOException;
  
  /** Creates an in-memory index with a specified strategy
   *  and appends all words extracted of a specific file.
   *  All words are appended with the line of the reader and the specifed filename. 
   *  
   *  Words are separated using the java whitespace rule
   *  <code>\p{javaWhitespace}</code> (see {@link Pattern})
   *  and word are indexed is its length is greater or equals to 3. 
   *  
   * @param strategy the context strategy used to create the index.
   * @param filename filename used for all word from the reader.
   * @param reader all words contained in the reader will be appended line by line
   *  in the index. 
   * @return a new in-memory index.
   * @throws IOException if the file doesn't exists or is unreadable.
   * 
   */
  public Index createIndex(String filename,Reader reader,ContextStrategy strategy) throws IOException;
  
  /** Creates an in-memory index with a specified strategy
   *  and appends all words extracted of a specific file in the specified index.
   *  All words are appended with the line of the reader and the specifed filename. 
   * @param strategy the context strategy used to create the index.
   * @param filename filename used for all word from the reader.
   * @param reader all words contained in the reader will be appended line by line
   *        in the index. 
   * @param accepter specifies how to separate word in the reader and
   *        which words are indexable.
   * @return a new in-memory index.
   * @throws IOException if the file doesn't exists or is unreadable.
   * 
   * @see 
   */
  public Index createIndex(String filename,Reader reader,WordAccepter accepter,ContextStrategy strategy) throws IOException;

  
  /** Loads an index stored in a file in memory.
   * @param indexFile file that store the index.
   * @return an in-memory index corresponding to the file.
   * @throws IOException raised if there is input/output problems.
   */
  public Index loadIndex(File indexFile)  throws IOException;
  
  /** Saves an index in a spacific file.
   * @param index the index to store.
   * @param indexFile file that will store the index.
   * @throws IOException raised if there is input/output problems.
   */
  public void saveIndex(Index index,File indexFile) throws IOException;
  
  /** Returns a query factory specific to an index.
   * @param index the queries will be created on this index.
   * @return a new query factory.
   * 
   * @see Query#execute(int)
   */
  public QueryFactory createQueryFactory(Index index);
  
  /** Returns all context strategies understood by the facade.
   *  This method must returned at least the "none" context strategy.
   * @return an unmodifiable map of context strategies associated
   *  with their names.
   */
  public Map<String,? extends ContextStrategy> getContextStrategyMap();  
}
